/**********************************************************************
 * PTlink IRC Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                     http://software.pt-link.net                    *
 * This program is distributed under GNU Public License               *
 * Please read the file COPYING for copyright information.            
 **********************************************************************

  Description: Memoserv send command

 *  $Id: ms_send.c,v 1.8 2005/10/18 16:25:06 jpinto Exp $
*/

#include "module.h"
#include "memoserv.h"
#include "nickserv.h" /* need IsAuthenticated() */
#include "my_sql.h"
#include "dbconf.h"
#include "nsmacros.h"
/* lang files */
#include "lang/common.lh"
#include "lang/ms_send.lh"

SVS_Module mod_info =
 /* module, version, description */
{"ms_send", "2.1",  "memoserv send command" };

/* Change Log
  2.1 - #35: ability to set order on ajoins
  2.0 - 0000265: remove nickserv cache system
*/

/** functions and events we require **/
/* void (*FunctionPointer)(void);*/
ServiceUser* (*memoserv_suser)(void);

MOD_REQUIRES
  DBCONF_FUNCTIONS
  MOD_FUNC(memoserv_suser)
MOD_END

/** Internal functions declaration **/
/* void internal_function(void); */
void ms_send(IRC_User *s, IRC_User *u);
int memos_count(u_int32_t snid);
u_int32_t insert_memo(char* sender_name, u_int32_t sender_snid, u_int32_t owner_snid, char* message);

    
int MaxMemosPerUser;
int NickSecurityCode;

DBCONF_REQUIRES
  DBCONF_GET("memoserv", MaxMemosPerUser)
  DBCONF_GET("nickserv", NickSecurityCode)
DBCONF_END

int mod_rehash(void)
{
  if(dbconf_get(dbconf_requires) < 0 )
  {
    errlog("Error reading dbconf!");
    return -1;
  }
  return 0;
} 

/** Local variables **/
/* int my_local_variable; */
ServiceUser* msu;
    
/** load code **/
int mod_load(void)
{
  msu = memoserv_suser();
  suser_add_cmd(msu, "SEND", ms_send, SEND_SUMMARY, SEND_HELP);
  return 0;
}
    
/** unload code **/
void mod_unload(void)
{
  suser_del_mod_cmds(msu, &mod_info);
}

/** internal functions implementation starts here **/
void ms_send(IRC_User *s, IRC_User *u)
{
  u_int32_t source_snid;
  u_int32_t snid;
  u_int32_t id;
  char* target;
  char* message;
  
  
  /* status validation */
  CHECK_IF_IDENTIFIED_NICK
  
  target = strtok(NULL, " ");
  message =  strtok(NULL, "");

  if(NickSecurityCode  && !IsAuthenticated(u))
    send_lang(u, s, NEEDS_AUTH_NICK);
  /* syntax validation */
  else if(IsNull(target) || IsNull(message))
    send_lang(u, s, SEND_SYNTAX);
  /* check requirements */
  else if((snid = nick2snid(target)) == 0)
    send_lang(u, s, NICK_X_NOT_REGISTERED, target);  
  else if(memos_count(snid) > MaxMemosPerUser)
    send_lang(u, s, MAX_MEMOS_REACHED_X_X, target, MaxMemosPerUser);
  /* execute operation */
  else if((id = insert_memo(u->nick, source_snid, snid, message)) > 0)
    {
      IRC_User* tu;
      send_lang(u, s, SENT_MEMO_TO_X, target);
      tu = irc_FindUser(target);
      if(tu && tu->snid) /* target is online and identified */
        {
          char memoprev[MEMOPREVMAX+1];
          snprintf(memoprev, MEMOPREVMAX, "%s", message);
          send_lang(tu, s, YOU_GOT_MEMO_FROM_X_X_NUM_X,
            u->nick, memoprev, id);
        }
    }
  else
    send_lang(u, s, UPDATE_FAIL);
}

int memos_count(u_int32_t snid)
{
  if(sql_singlequery("SELECT count(*) FROM memoserv"
    " WHERE owner_snid=%d", snid) > 0)
      return atoi(sql_field(0));
  return 0;  
}    

/* inserts a memo on the memoserv table */
u_int32_t insert_memo(char* sender_name, u_int32_t sender_snid, u_int32_t owner_snid, char* message)
{
  int r;
  u_int32_t max = 1;
  if((sql_singlequery("SELECT MAX(id) FROM memoserv"
    " WHERE owner_snid=%d ORDER BY id DESC LIMIT 1", owner_snid)  > 0) && sql_field(0))
      max = atoi(sql_field(0))+1;
  r = sql_execute("INSERT INTO memoserv VALUES(%d,"
    "%d, %d, %s, %d, %d, %s)",
    max, owner_snid, sender_snid, sql_str(sender_name), MFL_UNREAD, 
    time(NULL), sql_str(message));
  return r ? max : 0;
}


/* End of module */

